@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2020 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #ifndef COLOR_PALETTE_HPP
 #define COLOR_PALETTE_HPP
 
@@ -32,6 +57,7 @@ public:
 	// ADPCM sample editor
 	QColor instADPCMMemAllColor, instADPCMMemCurColor, instADPCMMemBackColor;
 	QColor instADPCMSampViewForeColor, instADPCMSampViewBackColor, instADPCMSampViewCenterColor;
+	QColor instADPCMSampViewGridColor, instADPCMSampViewDrawColor, instADPCMSampViewDirectDrawColor;
 
 	// Tone/Noise editor
 	QColor tnToneCellColor, tnToneTextColor;
@@ -48,8 +74,9 @@ public:
 	QColor odrHovCellColor;
 	QColor odrRowNumColor;
 	QColor odrHeaderTextColor, odrHeaderRowColor;
-	QColor odrBorderColor;
+	QColor odrBorderColor, odrHeaderBorderColor;
 	QColor odrBackColor;
+	QColor odrUnfocusedShadowColor;
 
 	// Pattern editor
 	QColor ptnDefTextColor, ptnDefStepColor, ptnHl1StepColor, ptnHl2StepColor;
@@ -62,10 +89,11 @@ public:
 	QColor ptnErrorColor;
 	QColor ptnHeaderTextColor, ptnHeaderRowColor;
 	QColor ptnMaskColor;
-	QColor ptnBorderColor;
+	QColor ptnBorderColor, ptnHeaderBorderColor;
 	QColor ptnMuteColor, ptnUnmuteColor;
 	QColor ptnBackColor;
 	QColor ptnMarkerColor;
+	QColor ptnUnfocusedShadowColor;
 
 	// Wave visual
 	QColor wavBackColor;
