@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #include "fm_operator_table.hpp"
 #include "ui_fm_operator_table.h"
 #include <QString>
@@ -6,8 +31,11 @@
 #include <QClipboard>
 #include "gui/event_guard.hpp"
 
-const int FMOperatorTable::DT_SIGN_TBL_[8] = { 0, 1, 2, 3, 0, -1, -2, -3 };
-const int FMOperatorTable::DT_UNSIGN_TBL_[7] = { 7, 6, 5, 0, 1, 2, 3 };
+namespace
+{
+constexpr int DT_SIGN_TBL_[8] = { 0, 1, 2, 3, 0, -1, -2, -3 };
+constexpr int DT_UNSIGN_TBL_[7] = { 7, 6, 5, 0, 1, 2, 3 };
+}
 
 FMOperatorTable::FMOperatorTable(QWidget *parent) :
 	QFrame(parent),
@@ -80,8 +108,6 @@ FMOperatorTable::FMOperatorTable(QWidget *parent) :
 		if (!isIgnoreEvent_) emit operatorValueChanged(Ui::FMOperatorParameter::SSGEG, value);
 	});
 
-	envmap_ = std::make_unique<QPixmap>();
-
 	ui->envFrame->installEventFilter(this);
 }
 
@@ -163,18 +189,16 @@ QString FMOperatorTable::toString() const
 	if (isDTNegative_) dt = DT_UNSIGN_TBL_[dt + 3];
 
 	auto str = QString("%1,%2,%3,%4,%5,%6,%7,%8,%9,%10")
-			   .arg(QString::number(ui->arSlider->value()))
-			   .arg(QString::number(ui->drSlider->value()))
-			   .arg(QString::number(ui->srSlider->value()))
-			   .arg(QString::number(ui->rrSlider->value()))
-			   .arg(QString::number(ui->slSlider->value()))
-			   .arg(QString::number(ui->tlSlider->value()))
-			   .arg(QString::number(ui->ksSlider->value()))
-			   .arg(QString::number(ui->mlSlider->value()))
-			   .arg(QString::number(dt))
-			   .arg(ui->ssgegCheckBox->isChecked()
-					? QString::number(ui->ssgegSlider->value())
-					: "-1");
+			   .arg(ui->arSlider->value())
+			   .arg(ui->drSlider->value())
+			   .arg(ui->srSlider->value())
+			   .arg(ui->rrSlider->value())
+			   .arg(ui->slSlider->value())
+			   .arg(ui->tlSlider->value())
+			   .arg(ui->ksSlider->value())
+			   .arg(ui->mlSlider->value())
+			   .arg(dt)
+			   .arg(ui->ssgegCheckBox->isChecked() ? ui->ssgegSlider->value() : -1);
 	return str;
 }
 
@@ -184,7 +208,7 @@ bool FMOperatorTable::eventFilter(QObject* obj, QEvent* event)
 		if (event->type() == QEvent::Paint) {
 			QPainter painter(ui->envFrame);
 			painter.eraseRect(ui->envFrame->rect());
-			painter.drawPixmap(ui->envFrame->rect(), *envmap_.get(), envmap_->rect());
+			painter.drawPixmap(ui->envFrame->rect(), envmap_, envmap_.rect());
 		}
 	}
 
@@ -207,9 +231,9 @@ void FMOperatorTable::resizeEvent(QResizeEvent* event)
 
 void FMOperatorTable::resizeGraph()
 {
-	envmap_ = std::make_unique<QPixmap>(ui->envFrame->size());
-	xr_ = (envmap_->width() - (ENV_LINE_W_ + 1) * 2.) / ENV_W_;
-	yr_ = (envmap_->height() - (ENV_LINE_W_ + 1) * 2.) / ENV_H_;
+	envmap_ = QPixmap(ui->envFrame->size());
+	xr_ = (envmap_.width() - (ENV_LINE_W_ + 1) * 2.) / ENV_W_;
+	yr_ = (envmap_.height() - (ENV_LINE_W_ + 1) * 2.) / ENV_H_;
 }
 
 void FMOperatorTable::repaintGraph()
@@ -313,8 +337,8 @@ void FMOperatorTable::repaintGraph()
 	p3.setY((127. - p3.y()) * envHrate);
 	p4.setY((127. - p4.y()) * envHrate);
 
-	envmap_->fill(palette_->instFMEnvBackColor);
-	QPainter painter(envmap_.get());
+	envmap_.fill(palette_->instFMEnvBackColor);
+	QPainter painter(&envmap_);
 
 	painter.setPen(QPen(palette_->instFMEnvGridColor, ENV_LINE_T_));
 	drawLine(painter, p1.x(), 0, p1.x(), envHeight);
