@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #include "instrument_form_manager.hpp"
 #include <utility>
 #include <QApplication>
@@ -6,9 +31,7 @@
 #include "gui/instrument_editor/instrument_editor_ssg_form.hpp"
 #include "gui/instrument_editor/instrument_editor_adpcm_form.hpp"
 #include "gui/instrument_editor/instrument_editor_drumkit_form.hpp"
-#include "misc.hpp"
-
-InstrumentFormManager::InstrumentFormManager() {}
+#include "utils.hpp"
 
 void InstrumentFormManager::updateByConfiguration()
 {
@@ -161,8 +184,7 @@ InstrumentType InstrumentFormManager::getFormInstrumentType(int n) const
 int InstrumentFormManager::checkActivatedFormNumber() const
 {
 	const QWidget* win = QApplication::activeWindow();
-	auto it = std::find_if(map_.begin(), map_.end(),
-						   [win](const std::pair<const int, std::shared_ptr<QWidget>> p) {
+	auto it = utils::findIf(map_, [win](const std::pair<const int, std::shared_ptr<QWidget>> p) {
 		return p.second.get() == win;
 	});
 
