@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #ifndef INSTRUMENT_FORM_MANAGER_HPP
 #define INSTRUMENT_FORM_MANAGER_HPP
 
@@ -6,16 +31,15 @@
 #include <memory>
 #include <unordered_map>
 #include "instrument/envelope_fm.hpp"
+#include "bamboo_tracker_defs.hpp"
 
 enum class InstrumentType;
-enum class SoundSource;
 
 class InstrumentFormManager : public QObject
 {
 	Q_OBJECT
 
 public:
-	InstrumentFormManager();
 	void updateByConfiguration();
 
 	const std::shared_ptr<QWidget> getForm(int n) const;
