@@ -1,10 +1,34 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #ifndef MAINWINDOW_HPP
 #define MAINWINDOW_HPP
 
 #include <atomic>
 #include <memory>
 #include <cstdint>
-#include <unordered_map>
 #include <QMainWindow>
 #include <QKeyEvent>
 #include <QListWidgetItem>
@@ -24,22 +48,20 @@
 #include <QShortcut>
 #include <QAction>
 #include <QActionGroup>
+#include <QLineEdit>
+#include "enum_hash.hpp"
 #include "configuration.hpp"
 #include "bamboo_tracker.hpp"
-#include "audio_stream.hpp"
+#include "precise_timer.hpp"
+#include "audio/audio_stream.hpp"
 #include "gui/instrument_editor/instrument_form_manager.hpp"
 #include "gui/color_palette.hpp"
-#include "timer.hpp"
 #include "gui/file_history.hpp"
 #include "gui/effect_list_dialog.hpp"
 #include "gui/keyboard_shortcut_list_dialog.hpp"
 #include "gui/bookmark_manager_form.hpp"
 #include "gui/instrument_selection_dialog.hpp"
 #include "gui/comment_edit_dialog.hpp"
-#include "file_io_error.hpp"
-#include "enum_hash.hpp"
-
-class AbstractBank;
 
 namespace Ui {
 	class MainWindow;
@@ -79,7 +101,7 @@ private:
 	std::shared_ptr<ColorPalette> palette_;
 	std::shared_ptr<BambooTracker> bt_;
 	std::shared_ptr<AudioStream> stream_;
-	std::unique_ptr<Timer> timer_;
+	std::unique_ptr<PreciseTimer> tickTimerForRealChip_;
 	std::unique_ptr<QTimer> visualTimer_;
 	std::shared_ptr<QUndoStack> comStack_;
 	std::shared_ptr<FileHistory> fileHistory_;
@@ -88,12 +110,15 @@ private:
 
 	// Instrument list
 	std::shared_ptr<InstrumentFormManager> instForms_;
+	QListWidgetItem* renamingInstItem_;
+	QLineEdit* renamingInstEdit_;
 	void addInstrument();
 	void addDrumkit();
 	void removeInstrument(int row);
 	void openInstrumentEditor();
 	int findRowFromInstrumentList(int instNum);
 	void renameInstrument();
+	void finishRenamingInstrument();
 	void cloneInstrument();
 	void deepCloneInstrument();
 	void loadInstrument();
@@ -146,8 +171,6 @@ private:
 	QString getModuleFileBaseName() const;
 	int getSelectedFileFilter(QString& file, QStringList& filters) const;
 
-	void showFileIOErrorDialog(const FileIOError& e, const QString sub = "");
-
 	bool isEditedPattern_, isEditedOrder_, isEditedInstList_;
 	bool isSelectedPattern_, isSelectedOrder_;
 
@@ -162,25 +185,18 @@ private:
 
 	// Toolbar
 	QSpinBox *octave_, *highlight1_, *highlight2_, *volume_;
-	using ToolbarPos = Configuration::ToolbarConfiguration::ToolbarPosition;
-	const std::unordered_map<ToolbarPos, Qt::ToolBarArea> TB_POS_ = {
-		{ToolbarPos::TOP_POS, Qt::TopToolBarArea },
-		{ ToolbarPos::BOTTOM_POS, Qt::BottomToolBarArea },
-		{ ToolbarPos::LEFT_POS, Qt::LeftToolBarArea },
-		{ ToolbarPos::RIGHT_POS, Qt::RightToolBarArea }
-	};
 
 	// Status bars
 	QLabel *statusDetail_, *statusStyle_, *statusInst_, *statusOctave_;
 	QLabel *statusIntr_, *statusMixer_, *statusBpm_, *statusPlayPos_;
 
 	// Shortcuts
-	std::unique_ptr<QAction> octUpSc_, octDownSc_;
-	std::unique_ptr<QShortcut> focusPtnSc_, focusOdrSc_, focusInstSc_, instAddSc_;
-	std::unique_ptr<QAction> playAndStopSc_, playStepSc_, goPrevOdrSc_, goNextOdrSc_;
-	std::unique_ptr<QAction> prevInstSc_, nextInstSc_, incPtnSizeSc_, decPtnSizeSc_;
-	std::unique_ptr<QAction> incEditStepSc_, decEditStepSc_, prevSongSc_, nextSongSc_;
-	std::unique_ptr<QAction> jamVolUpSc_, jamVolDownSc_;
+	QAction octUpSc_, octDownSc_;
+	QShortcut focusPtnSc_, focusOdrSc_, focusInstSc_;
+	std::unique_ptr<QShortcut> instAddSc_;
+	QAction playAndStopSc_, playStepSc_, goPrevOdrSc_, goNextOdrSc_, prevInstSc_, nextInstSc_;
+	QAction incPtnSizeSc_, decPtnSizeSc_, incEditStepSc_, decEditStepSc_, prevSongSc_, nextSongSc_;
+	QAction jamVolUpSc_, jamVolDownSc_;
 	void setShortcuts();
 
 	// Dialogs
@@ -193,13 +209,18 @@ private:
 	std::atomic_bool bankJamMidiCtrl_;
 	std::unique_ptr<InstrumentSelectionDialog> importBankDiag_;
 
+	// Track visibility
+	void setTrackVisibility(const std::vector<int>& visTracks);
+
 	// Meta methods
 	int tickEventMethod_;
 	int midiKeyEventMethod_;
 	int midiProgramEventMethod_;
 
 	void updateInstrumentListColors();
+	void setOrderListGroupMaximumWidth();
 	void freezeViews();
+	void unfreezeViews();
 
 	inline bool showUndoResetWarningDialog(QString text)
 	{
@@ -209,10 +230,27 @@ private:
 									 QMessageBox::No) == QMessageBox::Yes);
 	}
 
-	inline void showStreamFailedDialog()
+	inline void showStreamFailedDialog(const QString& errDetail)
 	{
 		QMessageBox::critical(this, tr("Error"),
-							  tr("Could not open the audio stream. Please change the sound settings in Configuration."),
+							  tr("Could not open the audio stream. Please change the sound settings in Configuration.")
+							  + "\n\n" + errDetail,
+							  QMessageBox::Ok, QMessageBox::Ok);
+	}
+
+	inline void showStreamRateWarningDialog(uint32_t curRate)
+	{
+		QMessageBox::warning(this, tr("Warning"),
+							 tr("Could not set the sample rate of the audio stream to %1Hz. Currently the stream runs on %2Hz instead.")
+							 .arg(config_.lock()->getSampleRate()).arg(curRate),
+							 QMessageBox::Ok, QMessageBox::Ok);
+	}
+
+	inline void showMidiFailedDialog(const QString& errDetail)
+	{
+		QMessageBox::critical(this, tr("Error"),
+							  tr("Could not initialize MIDI input.")
+							  + "\n\n" + errDetail,
 							  QMessageBox::Ok, QMessageBox::Ok);
 	}
 
@@ -321,6 +359,8 @@ private slots:
 	void on_action_Transpose_Song_triggered();
 	void on_action_Swap_Tracks_triggered();
 	void on_action_Insert_triggered();
+	void on_action_Hide_Tracks_triggered();
+	void on_action_Estimate_Song_Length_triggered();
 };
 
 #endif // MAINWINDOW_HPP
