@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2020 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #ifndef ORDER_LIST_EDITOR_HPP
 #define ORDER_LIST_EDITOR_HPP
 
@@ -25,6 +50,7 @@ public:
 	void setCommandStack(std::weak_ptr<QUndoStack> stack);
 	void setConfiguration(std::shared_ptr<Configuration> config);
 	void setColorPallete(std::shared_ptr<ColorPalette> palette);
+	void addActionToPanel(QAction* action);
 
 	void changeEditable();
 	void updatePositionByOrderUpdate(bool isFirstUpdate);
@@ -44,9 +70,10 @@ public:
 	void setFonts(QString headerFont, int headerSize, QString rowsFont, int rowsSize);
 
 	void setHorizontalScrollMode(bool cellBased, bool refresh = true);
+	void setVisibleTracks(std::vector<int> tracks);
 
 signals:
-	void currentTrackChanged(int num);
+	void currentTrackChanged(int idx);
 	void currentOrderChanged(int num);
 
 	void orderEdited();
@@ -56,8 +83,8 @@ signals:
 	void selected(bool isSelected);
 
 public slots:
-	void setCurrentTrack(int num);
-	void setCurrentOrder(int num, int max);
+	void onPatternEditorCurrentTrackChanged(int idx);
+	void onPatternEditorCurrentOrderChanged(int num, int max);
 	void onSongLoaded();
 	void onShortcutUpdated();
 
@@ -80,7 +107,7 @@ public slots:
 	void onOrderDataGlobalChanged();
 
 protected:
-	bool eventFilter(QObject *watched, QEvent *event) override;
+	bool eventFilter(QObject* watched, QEvent* event) override;
 	void resizeEvent(QResizeEvent* event) override;
 
 private:
@@ -92,6 +119,7 @@ private:
 
 	bool hScrollCellMove_;
 	void updateHorizontalSliderMaximum();
+	void updateMaximumWidth();
 };
 
 #endif // ORDER_LIST_EDITOR_HPP
