@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2020 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #include "pattern_editor.hpp"
 #include "ui_pattern_editor.h"
 
@@ -29,7 +54,7 @@ PatternEditor::PatternEditor(QWidget *parent) :
 		}
 	});
 	QObject::connect(ui->panel, &PatternEditorPanel::currentTrackChanged,
-					 this, [&](int num) { emit currentTrackChanged(num); });
+					 this, [&](int idx) { emit currentTrackChanged(idx); });
 	QObject::connect(ui->panel, &PatternEditorPanel::currentOrderChanged,
 					 this, [&](int num, int max) { emit currentOrderChanged(num, max); });
 
@@ -90,6 +115,11 @@ void PatternEditor::setColorPallete(std::shared_ptr<ColorPalette> palette)
 	ui->panel->setColorPallete(palette);
 }
 
+void PatternEditor::addActionToPanel(QAction* action)
+{
+	ui->panel->addAction(action);
+}
+
 void PatternEditor::changeEditable()
 {
 	ui->panel->changeEditable();
@@ -122,14 +152,15 @@ void PatternEditor::cutSelectedCells()
 
 void PatternEditor::freeze()
 {
+	setUpdatesEnabled(false);
 	freezed_ = true;
-	ui->panel->freeze();
+	ui->panel->waitPaintFinish();
 }
 
 void PatternEditor::unfreeze()
 {
 	freezed_ = false;
-	ui->panel->unfreeze();
+	setUpdatesEnabled(true);
 }
 
 QString PatternEditor::getHeaderFont() const
@@ -163,6 +194,18 @@ void PatternEditor::setHorizontalScrollMode(bool cellBased, bool refresh)
 	if (refresh) updateHorizontalSliderMaximum();
 }
 
+void PatternEditor::setVisibleTracks(std::vector<int> tracks)
+{
+	ui->horizontalScrollBar->setMaximum(200);	// Dummy
+	ui->panel->setVisibleTracks(tracks);
+	updateHorizontalSliderMaximum();
+}
+
+std::vector<int> PatternEditor::getVisibleTracks() const
+{
+	return ui->panel->getVisibleTracks();
+}
+
 bool PatternEditor::eventFilter(QObject *watched, QEvent *event)
 {
 	Q_UNUSED(watched)
@@ -222,14 +265,14 @@ void PatternEditor::resizeEvent(QResizeEvent* event)
 }
 
 /********** Slots **********/
-void PatternEditor::setCurrentTrack(int num)
+void PatternEditor::onOrderListCurrentTrackChanged(int idx)
 {
-	ui->panel->setCurrentTrack(num);
+	ui->panel->onOrderListCurrentTrackChanged(idx);
 }
 
-void PatternEditor::setCurrentOrder(int num)
+void PatternEditor::onOrderListCrrentOrderChanged(int num)
 {
-	ui->panel->setCurrentOrder(num);
+	ui->panel->onOrderListCurrentOrderChanged(num);
 }
 
 void PatternEditor::onOrderListEdited()
@@ -323,12 +366,12 @@ void PatternEditor::onChangeValuesPressed(bool isCoarse, bool isIncreased)
 
 void PatternEditor::onToggleTrackPressed()
 {
-	ui->panel->onToggleTrackPressed(ui->panel->getCurrentTrack());
+	ui->panel->onToggleTrackPressed();
 }
 
 void PatternEditor::onSoloTrackPressed()
 {
-	ui->panel->onSoloTrackPressed(ui->panel->getCurrentTrack());
+	ui->panel->onSoloTrackPressed();
 }
 
 void PatternEditor::onExpandPressed()
