@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2020 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #ifndef PATTERN_EDITOR_HPP
 #define PATTERN_EDITOR_HPP
 
@@ -24,6 +49,7 @@ public:
 	void setCommandStack(std::weak_ptr<QUndoStack> stack);
 	void setConfiguration(std::shared_ptr<Configuration> config);
 	void setColorPallete(std::shared_ptr<ColorPalette> palette);
+	void addActionToPanel(QAction* action);
 
 	void changeEditable();
 	void updatePositionByStepUpdate(bool isFirstUpdate);
@@ -43,9 +69,11 @@ public:
 	void setFonts(QString headerFont, int headerSize, QString rowsFont, int rowsSize);
 
 	void setHorizontalScrollMode(bool cellBased, bool refresh = true);
+	void setVisibleTracks(std::vector<int> tracks);
+	std::vector<int> getVisibleTracks() const;
 
 signals:
-	void currentTrackChanged(int num);
+	void currentTrackChanged(int idx);
 	void currentOrderChanged(int num, int max);
 
 	void focusIn();
@@ -60,8 +88,8 @@ protected:
 	void resizeEvent(QResizeEvent* event) override;
 
 public slots:
-	void setCurrentTrack(int num);
-	void setCurrentOrder(int num);
+	void onOrderListCurrentTrackChanged(int idx);
+	void onOrderListCrrentOrderChanged(int num);
 
 	void onOrderListEdited();
 	void onDefaultPatternSizeChanged();
