@@ -1,7 +1,32 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #include "vgm_export_settings_dialog.hpp"
 #include "ui_vgm_export_settings_dialog.h"
-#include "export_handler.hpp"
 #include <QTextCodec>
+#include "io/export_io.hpp"
 
 VgmExportSettingsDialog::VgmExportSettingsDialog(QWidget *parent) :
 	QDialog(parent),
@@ -12,8 +37,8 @@ VgmExportSettingsDialog::VgmExportSettingsDialog(QWidget *parent) :
 	setWindowFlags(windowFlags() ^ Qt::WindowContextHelpButtonHint);
 
 	for (QRadioButton *button : {
-			ui->ym2608RadioButton, ui->ym2612RadioButton, ui->ym2203RadioButton,
-			ui->internalSsgRadioButton, ui->ay8910PsgRadioButton })
+		 ui->ym2608RadioButton, ui->ym2612RadioButton, ui->ym2203RadioButton, ui->noneFmRadioButton,
+		 ui->internalSsgRadioButton, ui->ay8910PsgRadioButton })
 		connect(button, &QAbstractButton::toggled,
 				this, &VgmExportSettingsDialog::updateSupportInformation);
 
@@ -85,82 +110,93 @@ QString VgmExportSettingsDialog::getNotes() const
 	return ui->notesPlainTextEdit->toPlainText();
 }
 
-GD3Tag VgmExportSettingsDialog::getGD3Tag() const
+io::GD3Tag VgmExportSettingsDialog::getGD3Tag() const
 {
-	GD3Tag tag;
+	io::GD3Tag tag;
 	QTextCodec* sjis = QTextCodec::codecForName("Shift-JIS");
 	std::string endNull = "";
 	endNull += '\0';
 	endNull += '\0';
 
+	const QByteArray trackNameEn = getTrackNameEnglish().toLatin1();
 	tag.trackNameEn = "";
-	for (auto c : getTrackNameEnglish().toLatin1()) {
+	for (const auto& c : trackNameEn) {
 		tag.trackNameEn += c;
 		tag.trackNameEn += '\0';
 	}
 	tag.trackNameEn += endNull;
 
+	const QByteArray trackNameJp = sjis->fromUnicode(getTrackNameJapanese());
 	tag.trackNameJp = "";
-	for (auto c : sjis->fromUnicode(getTrackNameJapanese())) {
+	for (const auto& c : trackNameJp) {
 		tag.trackNameJp += c;
 	}
 	tag.trackNameJp += endNull;
 
+	const QByteArray gameNameEn = getGameNameEnglish().toLatin1();
 	tag.gameNameEn = "";
-	for (auto c : getGameNameEnglish().toLatin1()) {
+	for (const auto& c : gameNameEn) {
 		tag.gameNameEn += c;
 		tag.gameNameEn += '\0';
 	}
 	tag.gameNameEn += endNull;
 
+	const QByteArray gameNameJp = sjis->fromUnicode(getGameNameJapanese());
 	tag.gameNameJp = "";
-	for (auto c : sjis->fromUnicode(getGameNameJapanese())) {
+	for (const auto& c : gameNameJp) {
 		tag.gameNameJp += c;
 	}
 	tag.gameNameJp += endNull;
 
+	const QByteArray systemNameEn = getSystemNameEnglish().toLatin1();
 	tag.systemNameEn = "";
-	for (auto c : getSystemNameEnglish().toLatin1()) {
+	for (const auto& c : systemNameEn) {
 		tag.systemNameEn += c;
 		tag.systemNameEn += '\0';
 	}
 	tag.systemNameEn += endNull;
 
+	const QByteArray systemNameJp = sjis->fromUnicode(getSystemNameJapanese());
 	tag.systemNameJp = "";
-	for (auto c : sjis->fromUnicode(getSystemNameJapanese())) {
+	for (const auto& c : systemNameJp) {
 		tag.systemNameJp += c;
 	}
 	tag.systemNameJp += endNull;
 
+	const QByteArray authorEn = getTrackAuthorEnglish().toLatin1();
 	tag.authorEn = "";
-	for (auto c : getTrackAuthorEnglish().toLatin1()) {
+	for (const auto& c : authorEn) {
 		tag.authorEn += c;
 		tag.authorEn += '\0';
 	}
 	tag.authorEn += endNull;
 
+	const QByteArray authorJp = sjis->fromUnicode(getTrackAuthorJapanese());
 	tag.authorJp = "";
-	for (auto c : sjis->fromUnicode(getTrackAuthorJapanese())) {
+	for (const auto& c : authorJp) {
 		tag.authorJp += c;
 	}
 	tag.authorJp += endNull;
 
+	const QByteArray releaseDate = getReleaseDate().toLatin1();
 	tag.releaseDate = "";
-	for (auto c : getReleaseDate().toLatin1()) {
+	for (const auto& c : releaseDate) {
 		tag.releaseDate += c;
 		tag.releaseDate += '\0';
 	}
 	tag.releaseDate += endNull;
 
+	const QByteArray vgmCreator = getVgmCreator().toLatin1();
 	tag.vgmCreator = "";
-	for (auto c : getVgmCreator().toLatin1()) {
+	for (const auto& c : vgmCreator) {
 		tag.vgmCreator += c;
 		tag.vgmCreator += '\0';
 	}
 	tag.vgmCreator += endNull;
 
+	const QByteArray notes = getNotes().toLatin1();
 	tag.notes = "";
-	for (auto c : getNotes().toLatin1()) {
+	for (const auto& c : notes) {
 		tag.notes += c;
 		tag.notes += '\0';
 	}
@@ -173,15 +209,17 @@ int VgmExportSettingsDialog::getExportTarget() const
 {
 	int target = 0;
 
-	if (ui->ym2612RadioButton->isChecked())
-		target |= Export_YM2612;
+	if (ui->ym2608RadioButton->isChecked())
+		target |= io::Export_YM2608;
+	else if (ui->ym2612RadioButton->isChecked())
+		target |= io::Export_YM2612;
 	else if (ui->ym2203RadioButton->isChecked())
-		target |= Export_YM2203;
+		target |= io::Export_YM2203;
 
 	if (ui->ay8910PsgRadioButton->isChecked())
-		target |= Export_AY8910Psg;
+		target |= io::Export_AY8910Psg;
 	else if (ui->ym2149PsgRadioButton->isChecked())
-		target |= Export_YM2149Psg;
+		target |= io::Export_YM2149Psg;
 
 	return target;
 }
@@ -191,21 +229,24 @@ void VgmExportSettingsDialog::updateSupportInformation()
 	int target = getExportTarget();
 	int channels;
 
-	int fm = target & Export_FmMask;
-	int ssg = target & Export_SsgMask;
+	int fm = target & io::Export_FmMask;
+	int ssg = target & io::Export_SsgMask;
 
 	switch (fm) {
 	default:
 		channels = 6;
 		break;
-	case Export_YM2203:
+	case io::Export_YM2203:
 		channels = 3;
 		break;
+	case io::Export_NoneFm:
+		channels = 0;
+		break;
 	}
 
-	bool haveSsg = fm == Export_YM2608 || fm == Export_YM2203 || ssg != Export_InternalSsg;
-	bool haveRhythm = fm == Export_YM2608;
-	bool haveAdpcm = fm == Export_YM2608;
+	bool haveSsg = fm == io::Export_YM2608 || fm == io::Export_YM2203 || ssg != io::Export_InternalSsg;
+	bool haveRhythm = fm == io::Export_YM2608;
+	bool haveAdpcm = fm == io::Export_YM2608;
 
 	ui->supportFmChannelsLabel->setText(QString::number(channels));
 	ui->supportSsgLabel->setText(haveSsg ? tr("Yes") : tr("No"));
