@@ -1,3 +1,28 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #include "./gui/mainwindow.hpp"
 #include <memory>
 #include <exception>
@@ -13,18 +38,19 @@
 #include "gui/q_application_wrapper.hpp"
 #include "gui/configuration_handler.hpp"
 
+namespace
+{
 // Localization
 static void setupTranslations();
 static QString findQtTranslationsDir();
 static QString findAppTranslationsDir();
+}
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
 	try {
-		QString filePath = (argc > 1) ? argv[argc - 1] : "";	// Last argument file
-
 		std::shared_ptr<Configuration> config = std::make_shared<Configuration>();
-		ConfigurationHandler::loadConfiguration(config);
+		io::loadConfiguration(config);
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
 		QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
@@ -33,11 +59,14 @@ int main(int argc, char *argv[])
 		std::unique_ptr<QApplicationWrapper> a(std::make_unique<QApplicationWrapper>(argc, argv));
 		if (config->getEnableTranslation()) setupTranslations();
 		a->setWindowIcon(QIcon(":/icon/app_icon"));
+
+		QString filePath = (argc > 1) ? argv[argc - 1] : "";	// Last argument file
+
 		std::unique_ptr<MainWindow> w(std::make_unique<MainWindow>(config, filePath));
 		w->show();
 		int ret = a->exec();
 
-		ConfigurationHandler::saveConfiguration(config);
+		io::saveConfiguration(config);
 		if (ret) QMessageBox::critical(nullptr, QObject::tr("Error"), QObject::tr("An unknown error occurred."));
 
 		return ret;
@@ -49,8 +78,10 @@ int main(int argc, char *argv[])
 	}
 }
 
+namespace
+{
 // Sets up the translation according to the current language
-static void setupTranslations()
+void setupTranslations()
 {
 	QApplication *a = qApp;
 	const QString lang = QLocale::system().name();
@@ -78,7 +109,7 @@ static void setupTranslations()
 }
 
 // Finds the location of Qt translation catalogs
-static QString findQtTranslationsDir()
+QString findQtTranslationsDir()
 {
 #if defined(Q_OS_DARWIN)
 	// if this is macOS, attempt to load from inside an app bundle
@@ -97,7 +128,7 @@ static QString findQtTranslationsDir()
 }
 
 // Finds the location of our translation catalogs
-static QString findAppTranslationsDir()
+QString findAppTranslationsDir()
 {
 #ifndef QT_NO_DEBUG
 	// if this is a debug build, attempt to load from the source directory
@@ -120,3 +151,4 @@ static QString findAppTranslationsDir()
 	return QApplication::applicationDirPath() + "/../share/BambooTracker/lang";
 #endif
 }
+}
