@@ -1,8 +1,33 @@
+/*
+ * Copyright (C) 2018-2021 Rerrah
+ *
+ * Permission is hereby granted, free of charge, to any person
+ * obtaining a copy of this software and associated documentation
+ * files (the "Software"), to deal in the Software without
+ * restriction, including without limitation the rights to use,
+ * copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following
+ * conditions:
+ *
+ * The above copyright notice and this permission notice shall be
+ * included in all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+ * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
+ * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+ * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
+ * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
+ * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
+ * OTHER DEALINGS IN THE SOFTWARE.
+ */
+
 #pragma once
 
 #include <cstdint>
 #include <string>
-#include "misc.hpp"
+#include <stdexcept>
 
 class Version
 {
@@ -26,7 +51,7 @@ private:
 	// Application version
 	static constexpr unsigned int appMajor			= 0;
 	static constexpr unsigned int appMinor			= 4;
-	static constexpr unsigned int appRevision		= 2;
+	static constexpr unsigned int appRevision		= 6;
 
 	// Module file version
 	static constexpr unsigned int modFileMajor		= 1;
@@ -43,9 +68,12 @@ private:
 	static constexpr unsigned int bankFileMinor		= 2;
 	static constexpr unsigned int bankFileRevision	= 0;
 
-	Version() {}
+	Version() = default;
+
+	static uint8_t uitobcd(const uint8_t v);
 };
 
+//===============================================
 inline uint32_t Version::ofApplicationInBCD()
 {
 	return toBCD(appMajor, appMinor, appRevision);
@@ -98,3 +126,12 @@ inline std::string Version::toString(unsigned int major, unsigned int minor, uns
 {
 	return std::to_string(major) + "." + std::to_string(minor) + "." + std::to_string(revision);
 }
+
+inline uint8_t Version::uitobcd(const uint8_t v)
+{
+	if (v > 99) throw std::out_of_range("Out of range.");
+
+	uint8_t high = v / 10;
+	uint8_t low = v % 10;
+	return static_cast<uint8_t>(high << 4) + low;
+}
